%--------------------------------------------------------------------------------
% General function to calculate a window function (mostly a wrapper
% for the existing window functions).
% 
% USE: win = get_window( win_length, win_type, [win_param], [DFT_WINDOW], [Npad] )
%
% INPUT:
%       win_length = length of window
%       win_type   = type of window: { 'delta' | 'rect' | 'bart' | 'hamm'
%                    | 'hann' | 'tukey' | 'gauss' | 'cosh' }
%       win_param  = (optional) window parameter.
%       DFT_WINDOW = (optional) parameter { 0 | 1 }. 
%                     If 1 returns DFT of window. 
%       Npad       = (optional) zero-pad window to length Npad.
%
% OUTPUT:
%       win = window 
%
% EXAMPLE
%      N=64; 
%      win=get_window(N,'tukey',0.4,0,2*N); 
%      plot(win);
%
%
% If you use this code in your research or publications, please
% reference [1].
%
%
% [1] J.M. O' Toole, M. Mesbah, B. Boashash, "Accurate and efficient
% implementation of the time-frequency matched filter", IET Signal
% Process., Special Issue of Time-Frequency Approach to Radar
% Detection, Imaging, and Classification, submitted for publication,
% 2009

%
%   Copyright (c) 2009, John M. O' Toole, The University of Queensland
%   All rights reserved.
%
%  Redistribution and use in source and binary forms, with or without
%  modification, are permitted provided that the following
%  conditions are met:
%      * Redistributions of source code must retain the above
%        copyright notice, this list of conditions and the following
%        disclaimer.
%      * Redistributions in binary form must reproduce the above
%        copyright notice, this list of conditions and the following
%        disclaimer in the documentation and/or other materials
%        provided with the distribution.
%      * Neither the name of the <organization> nor the names of its
%        contributors may be used to endorse or promote products
%        derived from this software without specific prior written
%        permission.
%  
%  THIS SOFTWARE IS PROVIDED BY JOHN M. O' TOOLE ''AS IS'' AND ANY
%  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
%  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JOHN M. O' TOOLE BE
%  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
%  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
%  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
%  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
%  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
%  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
%  DAMAGE.
%
%-------------------------------------------------------------------------------
function win = get_window( win_length, win_type, win_param, freq_domain_rep, Npad )
if( nargin<3 ) win_param=[]; end
if( nargin<4 ) freq_domain_rep=0; end
if( nargin<5 ) Npad=0; end


win=get_win(win_length,win_type,win_param,freq_domain_rep);
win=shift_win(win);

if(Npad>0)
  win=pad_win(win,Npad);
end


%--------------------------------------------------------------------------------
% Get the window. Negative indices are first.
%--------------------------------------------------------------------------------
function win=get_win(win_length, win_type, win_param, freq_domain_rep )

switch win_type
 case 'delta'
  win=zeros(win_length,1);
  wh = floor(win_length/2);
  win(wh+1)=1;
 case 'rect'
  win(1:win_length) = 1;
 case 'bart'
  win = bartlett( win_length );
 case 'hamm'
  win = hamming( win_length );
 case 'hann'
  win = hanning( win_length );
 case 'tukey'
  % NOTE: seems to be problem with Octave's (v2.9.12) tukeywin.m for N odd.
  if(( nargin < 3 ) | ( win_param == 'NULL' ))
    win = tukeywin( win_length );
  else
    win = tukeywin( win_length, win_param );
  end
 case 'gauss'
  if(( nargin < 3 ) | ( win_param == 'NULL' ))
    win = gausswin( win_length );
  else
    win = gausswin( win_length, win_param );
  end
 case 'cosh'
  win_hlf = fix( win_length / 2);

  if(( nargin < 3 ) | ( win_param == 'NULL' ))
    win_param = 0.01;
  end
  for m = -win_hlf:win_hlf
    win(mod(m,win_length)+1) = cosh( m ).^( -2 * win_param );
  end
  win = fftshift(win);
  
  
 otherwise
  error(['Unknown window type ' win_type]);
end
  


%---------------------------------------------------------------------
% If want the DFT of win
%---------------------------------------------------------------------
if( freq_domain_rep==1 )
  win=circshift(win(:),ceil(win_length/2));
  win=fft(win);
  win=circshift(win(:),floor(win_length/2));
end
  
  
%--------------------------------------------------------------------------------
% Shift the window so that positive indices are first.
%--------------------------------------------------------------------------------
function w=shift_win(w)
N=length(w);
w=circshift(w(:),ceil(N/2));



